
static char help[] = "Reads matrix and debug solver\n\n";
/*
 Example:

      This reproduces the problem:

      mpirun -n 2 ./ex16 -f binaryoutput -pc_type lu -pc_factor_mat_solver_package superlu_dist

*/

#include <petscksp.h>

#undef __FUNCT__
#define __FUNCT__ "main"
int main(int argc,char **args)
{
  Mat               A;
  PC                pc;
  PetscViewer       fd;                        /* viewer */
  char              file[PETSC_MAX_PATH_LEN];  /* input file name */
  PetscErrorCode    ierr;
  PetscInt          m,n;
  PetscBool         flg;
  PetscMPIInt       rank, size;

  PetscInitialize(&argc,&args,(char*)0,help);

  ierr = MPI_Comm_size(PETSC_COMM_WORLD,&size); CHKERRQ(ierr);
  ierr = MPI_Comm_rank(PETSC_COMM_WORLD,&rank); CHKERRQ(ierr);

  /* Determine files from which we read the linear systems. */
  ierr = PetscOptionsGetString(NULL,NULL,"-f",file,PETSC_MAX_PATH_LEN,&flg); CHKERRQ(ierr);
  if (!flg) SETERRQ(PETSC_COMM_WORLD,1,"Must indicate binary file with the -f option");

  ierr = MatCreate(PETSC_COMM_WORLD,&A); CHKERRQ(ierr);

  /* ad-hoc, since matview/load ignores partitioning */
  if(size == 2){

     if(rank==0) {m=n=8448;}
     if(rank==1) {m=n=8704;}

     ierr = MatSetSizes(A,m,n,PETSC_DECIDE,PETSC_DECIDE); CHKERRQ(ierr);

  }

  ierr = PCCreate(PETSC_COMM_WORLD, &pc); CHKERRQ(ierr);

  ierr = PCSetFromOptions(pc); CHKERRQ(ierr);

  /* firts iteration */
  PetscPrintf(PETSC_COMM_WORLD, "Works for the first time! \n");

  ierr = PetscViewerBinaryOpen(PETSC_COMM_WORLD,file,FILE_MODE_READ,&fd); CHKERRQ(ierr);

  ierr = MatLoad(A,fd); CHKERRQ(ierr);

  ierr = PetscViewerDestroy(&fd); CHKERRQ(ierr);

  ierr = PCSetOperators(pc,A,A); CHKERRQ(ierr);

  ierr = PCSetUp(pc); CHKERRQ(ierr);

  /* second iteration, same matrix, but pc structures already exist */
  PetscPrintf(PETSC_COMM_WORLD, "Crashes here! \n");

  ierr = PetscViewerBinaryOpen(PETSC_COMM_WORLD,file,FILE_MODE_READ,&fd); CHKERRQ(ierr);

  ierr = MatLoad(A,fd); CHKERRQ(ierr);

  ierr = PetscViewerDestroy(&fd); CHKERRQ(ierr);

  ierr = PCSetOperators(pc,A,A); CHKERRQ(ierr);

  ierr = PCSetUp(pc); CHKERRQ(ierr);

  ierr = MatDestroy(&A); CHKERRQ(ierr);
  ierr = PCDestroy(&pc); CHKERRQ(ierr);

  ierr = PetscFinalize();

  return 0;
}

